import Vue from 'vue';
import Router from 'vue-router';
import { Router as RouterDirect, Config, Http } from '@zeedhi/core';
import { TekLibLocalStorage } from '@zeedhi/tek-lib';
import { LoadingService } from '@zeedhi/common';
import { ZdPage } from '@zeedhi/vue';
import { ZdFramePage } from '@zeedhi/vuetify';
import { Utilities } from './util/Utilities';
import Store from './store';

Vue.use(Router);

const router = new Router({
    mode: 'history',
    base: process.env.BASE_URL,
    routes: [
        {
            path: '/',
            name: 'home',
            component: () => import('./views/Home.vue'),
        },
        {
            name: 'fm_00011_000_analise_nao_conformidades',
            path: '/fm_00011_000_analise_nao_conformidades',
            component: ZdPage,
            props: {
                path: 'fm_00011_000_analise_nao_conformidades/fm_00011_000_analise_nao_conformidades',
                local: true,
            },
        },
        {
            path: '/fm_00004_000_tarefas',
            name: 'Tarefas',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00004_000_tarefas/fm_00004_000_tarefas',
                    routePath: '/fm_00004_000_tarefas',
                },
            },
        },
        {
            name: 'DashboardSensor',
            path: '/DashboardSensor',
            component: ZdPage,
            props: {
                path: 'sensor/layout_dashboardSensor',
                local: true,
            },
        },
        {
            name: 'ContadorSentidoUnico',
            path: '/SentidoUnico',
            component: ZdPage,
            props: {
                path: 'sensor/layout_sentidoUnico',
                local: true,
            },
        },
        {
            name: 'ContadorSentidoDuplo',
            path: '/SentidoDuplo',
            component: ZdPage,
            props: {
                path: 'sensor/layout_sentidoDuplo',
                local: true,
            },
        },
        {
            name: 'ContadorSentidoGeral',
            path: '/SentidoGeral',
            component: ZdPage,
            props: {
                path: 'sensor/layout_sentidoGeral',
                local: true,
            },
        },
        {
            path: '/fm_00007_000_calendario',
            name: 'Calendario',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00007_000_calendario/fm_00007_000_calendario',
                    routePath: '/fm_00007_000_calendario',
                },
            },
        },
        {
            name: 'fm_0000_000_historico_transferencia',
            path: '/fm_0000_000_historico_transferencia',
            component: ZdPage,
            props: {
                path: 'fm_0000_000_historico_transferencia/fm_0000_000_historico_transferencia',
                local: true,
            },
        },
        {
            path: '/fm_00008_000_hierarquia_usuarios',
            name: 'hierarquiaUsuarios',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: '/fm_00008_000_hierarquia_usuarios/fm_00008_000_hierarquia_usuarios',
                    routePath: '/fm_00008_000_hierarquia_usuarios',
                },
            },
        },
        {
            path: '/zeedhi',
            name: 'zeedhi',
            component: ZdPage,
            props: () => ({
                path: 'zeedhi',
                local: true,
            }),
        },
        {
            path: '/auth/login',
            name: 'login',
            component: ZdPage,
            props: () => ({
                path: 'login',
                local: true,
            }),
        },
        {
            path: '*',
            component: ZdPage,
            props: {
                path: 'notfound',
                local: true,
            },
        },
        {
            path: '/fm_00003_000_checklist',
            name: 'Checklist',
            component: ZdPage,
            props: {
                path: 'fm_00003_000_checklist/fm_00003_000_checklist',
                local: true,
            },
        },
        {
            path: '/fm_00000_000_dashboard',
            name: 'Dashboard',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00000_000_dashboard/fm_00000_000_dashboard',
                    routePath: '/fm_00000_000_dashboard',
                },
            },
        },
        {
            path: '/FM_00002_000_gerenciar_locais',
            name: 'Gerenciar Locais',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00002_000_gerenciar_locais/FM_00002_000_gerenciar_locais',
                    routePath: '/FM_00002_000_gerenciar_locais',
                },
            },
        },
        {
            path: '/perfilDeAcesso',
            component: ZdPage,
            props: {
                path: 'perfilDeAcesso/perfilDeAcesso',
                local: true,
            },
        },
        {
            path: '/fm_00005_000_vistorias',
            name: 'Vistorias',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00005_000_vistorias/fm_00005_000_vistorias',
                    routePath: '/fm_00005_000_vistorias',
                },
            },
        },
        {
            path: '/Sensores',
            component: ZdPage,
            props: {
                path: 'parametrosSensor',
                local: true,
            },
        },
        {
            path: '/fm_070_000_parametros_sensor',
            component: ZdPage,
            props: {
                path: '/fm_070_000_parametros_sensor/fm_070_000_parametros_sensor',
                local: true,
            },
        },
        {
            path: '/grafico',
            component: ZdPage,
            props: {
                path: 'grafico',
                local: true,
            },
        },
        {
            path: '/hierarquias',
            component: ZdPage,
            props: {
                path: 'hierarquias',
                local: true,
            },
        },
        {
            path: '/gerarGrafico',
            component: ZdPage,
            props: {
                path: 'gerarGrafico',
                local: true,
            },
        },
        {
            path: '/hierarquias/detalhes/:id',
            component: ZdPage,
            props: {
                path: 'hierarquiaDetalhes',
                local: true,
            },
        },
        {
            path: '/fm_00020_000_solicitacoes',
            name: 'Solicitações',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00020_000_solicitacoes/fm_00020_000_solicitacoes',
                    routePath: '/fm_00020_000_solicitacoes',
                },
            },
        },
        {
            path: '/fm_00021_000_apontamento',
            name: 'Apontamento',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: '/fm_00021_000_apontamento/fm_00021_000_apontamento',
                    routePath: '/fm_00021_000_apontamento',
                },
            },
        },
        {
            name: 'Tarefa Recorrente',
            path: '/fm_00030_000_tarefas_recorrentes',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00030_000_tarefas_recorrentes/fm_00030_000_tarefas_recorrentes',
                    routePath: '/fm_00030_000_tarefas_recorrentes',
                },
            },
        },
        {
            path: '/fm_00010_000_parametrizao_geral',
            name: 'Parametrização Geral',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00010_000_parametrizao_geral/fm_00010_000_parametrizao_geral',
                    routePath: '/fm_00010_000_parametrizao_geral',
                },
            },
        },
        {
            path: '/fm_00028_002_relatorio_customizavel_vistorias',
            name: 'Relatorios Customizavel Vistorias',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00028_000_relatorios/fm_00028_002_relatorio_customizavel_vistorias',
                    routePath: '/fm_00028_002_relatorio_customizavel_vistorias',
                },
            },
        },
        {
            path: '/fm_00028_001_relatorio_customizavel_tarefas',
            name: 'Relatorios Customizavel Tarefas',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00028_000_relatorios/fm_00028_001_relatorio_customizavel_tarefas',
                    routePath: '/fm_00028_001_relatorio_customizavel_tarefas',
                },
            },
        },
        {
            name: 'Histórico de Transferência de Tarefas',
            path: '/fm_0000_000_historico_transferencia',
            component: ZdPage,
            props: {
                path: 'fm_0000_000_historico_transferencia/fm_0000_000_historico_transferencia',
                local: true,
            },
        },
        {
            path: '/fm_00031_000_tarefas_vistorias_contrato',
            name: 'Tarefas/Vistorias por Contrato',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00031_000_tarefas_vistorias_contrato/fm_00031_000_tarefas_vistorias_contrato',
                    routePath: '/fm_00031_000_tarefas_vistorias_contrato',
                },
            },
        },
        {
            path: '/fm_00028_003_relatorio_tarefas_atrasadas',
            name: 'Relatorios Tarefas Atrasadas',
            component: ZdFramePage,
            props: {
                type: 'post',
                params: {
                    jsonPath: 'fm_00028_000_relatorios/fm_00028_003_relatorio_tarefas_atrasadas',
                    routePath: '/fm_00028_003_relatorio_tarefas_atrasadas',
                },
            },
        },
        {
            path: '/modules/:module/:page',
            name: 'microservice',
            component: () => import('./views/Microservice.vue'),
        },
        {
            path: '*',
            component: ZdPage,
            props: {
                path: 'notfound',
                local: true,
            },
        },
    ],
});

const isOpeningNewModule = function isOpeningNewModule(to: any, from: any) {
    const isFromProduct = (to.name === 'microservice' && from.name !== 'microservice');
    // eslint-disable-next-line max-len
    const isFromAnotherModule = ((to.name === 'microservice' && from.name === 'microservice') && to.path.split('/')[2] !== from.path.split('/')[2]);
    const isNotLoginRoute = (to.fullPath !== Config.env.modules.login.routePath);
    return ((isFromProduct || isFromAnotherModule) && isNotLoginRoute);
};

// eslint-disable-next-line max-len
const callLibStartSessionFromModule = async function callLibStartSessionFromModule(to: any) {
    const data = {
        USER: localStorage.getItem('LOGIN_USER'),
        LOGIN_LANGUAGE: localStorage.getItem('LOGIN_LAGUAGE'),
        LOGIN_TOKEN: localStorage.getItem('LOGIN_TOKEN'),
    };
    await Http.post(`${Config.env.modules[to.path.split('/')[2]].endPoint}/lib_startSession`, data);
};

router.beforeEach(async (to, from, next) => {
    await Utilities.timeout(500);
    if (isOpeningNewModule(to, from)) {
        try {
            LoadingService.show();
            await callLibStartSessionFromModule(to);
            LoadingService.hide();
        } catch (e) {
            LoadingService.hide();
        }
    }

    /** open old application - init */
    if (window.parent !== window && to.name !== 'token-error' && to.name !== 'microservice') { // if application is called inside a iframe
        if (to.query.token) {
            // if token was passed
            localStorage.setItem('LOGIN_TOKEN', to.query.token.toString());
            localStorage.setItem('LOGIN_USER', to.query.user.toString());
            localStorage.setItem('LOGIN_LANGUAGE', to.query.languange.toString());
            localStorage.setItem('PRODUCT_ID', to.query.productId.toString());
            if (to.query.hash !== '0') {
                localStorage.setItem('LOGIN_HASH', to.query.hash.toString());
            }
            Store.commit('setAppLoginToken', to.query.token);
            Store.commit('setAppDisplayOptions', { menu: to.query.menu, header: to.query.header });
        }

        if (!Store.state!.appLoginToken) {
            // if there is no token... redirect to another page
            Store.commit('setAppDisplayOptions', { menu: '0', header: '0' });
            next('/token-error');
            return;
        }

        // remove parameters from requested page
        delete to.query.menu;
        delete to.query.header;
        delete to.query.token;

        TekLibLocalStorage.setItem('IS_NEXT', false);
        TekLibLocalStorage.setItem('LOGIN_KEEP_CONNECTED', 'Yes');
        TekLibLocalStorage.removeItem('routeParams');

        next();
    } else {
        TekLibLocalStorage.setItem('IS_NEXT', true);
    }
    /** open old application - end */

    if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
        next(Config.env.loginRoutePath);
    } else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
        next(Config.env.baseUrl);
    } else {
        if (to.path !== Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
            const dashboards: any = { O: '/dashboard', S: '/dashboardSupervisor', G: '/dashboardGerencial' };
            const rotaDash = dashboards[<string>localStorage.getItem('IDTIPODASHBOARD')];
            if (to.path.includes('dashboard') && to.path !== rotaDash) {
                RouterDirect.replace(rotaDash);
            }
            if (to.path !== '/localAtendimento/edit' && to.path !== '/localAtendimento/add') {
                localStorage.removeItem('dadosToEdit');
            } else {
                const dados = JSON.parse(<string>localStorage.getItem('dadosToEdit')) || [];
                if (dados.length > 0) {
                    dados.setLocalCli = true;
                }
                window.localStorage.setItem('dadosToEdit', JSON.stringify(dados));
            }
        }
        next();
    }
});

router.afterEach((to, from) => {
    const htmlElement = document.getElementsByTagName('html')[0];

    if (to.name === 'microservice') {
        htmlElement.setAttribute('microservice', '');
        Store.commit('setRouterParams', to.params);
        const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
        if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
            frame.contentWindow.postMessage({
                type: 'microservice-route-change',
                routerParams: to.params,
            }, `${Config.env.modules[to.path.split('/')[2]].baseUrl}`);
        }
        window.document.title = `${Config.env.product.name} - ${to.params.page}`;
    } else {
        htmlElement.removeAttribute('microservice');
        window.document.title = Config.env.product.name;
    }

    if (to.name === 'login') {
        htmlElement.setAttribute('login', '');
    } else {
        htmlElement.removeAttribute('login');
    }
});

export const { VUE_APP_BASE_URL } = process.env;

export const getRoutePath = () => {
    const { currentRoute } = router;
    const { path } = currentRoute;
    return path;
};

export const getRouteParams = () => {
    const { currentRoute } = router;
    const { params } = currentRoute;
    return params;
};

export default router;
